@@ -2,14 +2,14 @@
  *
  * mqmprob.cpp
  *
- * Copyright (c) 1996-2009 by
+ * Copyright (c) 1996-2010 by
  * Ritsert C Jansen, Danny Arends, Pjotr Prins and Karl W Broman
  *
  * initial MQM C code written between 1996-2002 by Ritsert C. Jansen
  * improved for the R-language by Danny Arends, Pjotr Prins and Karl W. Broman
  *
  * Modified by Danny Arends and Pjotr Prins
- * last modified December 2009
+ * last modified May 2010
  *
  *     This program is free software; you can redistribute it and/or
  *     modify it under the terms of the GNU General Public License,
@@ -28,6 +28,7 @@
  **********************************************************************/
 
 #include "mqm.h"
+#include <R_ext/Arith.h>
 
 /* This function walks the marker list and determins for every position whether the marker is in the Middle, Left,Right of the chromosome
  When there is only 1 marker on a chromosome it is defined Unlinked*/
@@ -163,7 +164,7 @@ double start_prob(const MQMCrossType crosstype, MQMMarker marker) {
       fatal("Strange: unknown crosstype in start_prob");
   }
   fatal("Should not get here");
-  return NAN;
+  return R_NaN;
 }
 
 
@@ -177,7 +178,7 @@ double left_prob(const double r,const MQMMarker markerL,const MQMMarker markerR,
   const double rr = 1.0-r;      // No recombination
   const double rr2 = rr*rr;     //Double Norecombination
  
-  const int recombinations = (int)fabs((double)markerL-(double)markerR);    //The number of recombinations
+  const int recombinations = abs(markerL-markerR);    //The number of recombinations
 
   switch (crosstype) {
     case CF2: //F2 cross
@@ -212,10 +213,10 @@ double left_prob(const double r,const MQMMarker markerL,const MQMMarker markerR,
       break;
     default:
       fatal("Strange: unknown crosstype in prob");
-      return NAN;
+      return R_NaN;
   }
   fatal("Should not get here");
-  return NAN;
+  return R_NaN;
 }
 
 /*
@@ -292,10 +293,10 @@ bool is_knownMarker(const char marker,const MQMCrossType crosstype){
     break;
     case CUNKNOWN:
       fatal("Strange: unknown crosstype in is_knownMarker()");
-      return NAN;
+      return R_NaN;
     break;
   }
-  return NAN;
+  return R_NaN;
 }
 
 /*
@@ -317,7 +318,7 @@ double right_prob_F2(const char markerL, const int j, const MQMMarkerVector imar
   const double r2 = r*r;      //Breeding Logic (see prob_new)
   const double rr = 1.0-r;
   const double rr2 = rr*rr;
-  const int recombinations = (int)fabs(markerL-markerR);   //Number of recombinations between markerL and markerR
+  const int recombinations = abs(markerL-markerR);   //Number of recombinations between markerL and markerR
 
   if (is_knownMarker(markerR, CF2)) {   //If we know the next marker we have an answer
     if ((markerL==MH)&&(markerR==MH)) {
@@ -391,7 +392,8 @@ double right_prob_BC(const char markerL, const int j, const MQMMarkerVector imar
   double prob0 = 0.0;                                  		//Internal variable holding the probability AA if the next rightmarker is (Semi) Unknown
   double prob1 = 0.0;                                  		//Internal variable holding the probability H if the next rightmarker is (Semi) Unknown
   const double rr = 1.0-r;                             		//Breeding Logic (see prob_new)
-  const int recombinations = (int)fabs(markerL-markerR);    //Number of recombinations between markerL and markerR
+  const int recombinations = abs(markerL-markerR);    //Number of recombinations between markerL and markerR
+
   if (is_knownMarker(markerR, CBC)) {
     return ((recombinations==0)? rr : r );
   } else {
@@ -419,7 +421,8 @@ double right_prob_RIL(const char markerL, const int j, const MQMMarkerVector ima
   double prob0 = 0.0;
   double prob2 = 0.0;
   const double rr = 1.0-r;
-  const int recombinations = (int)fabs(markerL-markerR);
+  const int recombinations = abs(markerL-markerR);
+
   if (is_knownMarker(markerR, CRIL)) {
     return ((recombinations==0) ? rr : r);
   } else { //Next marker is semi unknown
